using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.Events;

namespace RightClick
{
    public class RightClickLimbBehaviour : MonoBehaviour
    {
        public LimbBehaviour limb;
        public PersonBehaviour PB;
        public RightClickPersonBehaviour rcpb;

        void Start()
        {
            limb = gameObject.GetComponent<LimbBehaviour>();
            PB = limb.Person;
            rcpb = PB.GetComponent<RightClickPersonBehaviour>();

            var knockOut = new ContextMenuButton(() => PB.Consciousness == 1f, "knockOut", "Knock out", "Knock out the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    PB.Consciousness = 0f;
                    foreach (var limb in PB.Limbs)
                    {   
                        limb.Numbness = 1f;
                    }
                })
            });
            knockOut.LabelWhenMultipleAreSelected = "Knock out";

            var wakeUp = new ContextMenuButton(() => PB.Consciousness != 1f && PB.IsAlive(), "wakeUp", "Wake up", "Wake up the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
               
                    PB.OxygenLevel = 1f;
                    PB.Consciousness = 1f;
                    PB.ShockLevel = 0f;
                    PB.PainLevel = 0f;
                    PB.AdrenalineLevel = 0f;
                    
                    foreach (var limb in PB.Limbs) {
                        limb.Numbness = 0f;
                        limb.CirculationBehaviour.InternalBleedingIntensity = 0f;
                    }
                })
            });
            wakeUp.LabelWhenMultipleAreSelected = "Wake up";

            var kill = new ContextMenuButton(() => PB.IsAlive(), "kill", "Kill", "Kill the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    foreach (var limb in PB.Limbs)
                    {   
                        limb.CirculationBehaviour.IsPump = false;
                        limb.CirculationBehaviour.ForceSetAllLiquid(0f);
                    }
                })
            });
            kill.LabelWhenMultipleAreSelected = "Kill";

            var revive = new ContextMenuButton(() => rcpb.Invincibling == false, "revive", "Revive", "Revive the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {                           
                    foreach (var limb in PB.Limbs)
                    {
                        if (limb.IsAndroid) {
                            CirculationBehaviour circulationBehaviour = limb.CirculationBehaviour;
                            limb.Health = limb.InitialHealth;
                            limb.HealBone();
                            circulationBehaviour.HealBleeding();
                            circulationBehaviour.ClearLiquid();
                            circulationBehaviour.AddLiquid(limb.GetOriginalBloodType(), 1f);
                            circulationBehaviour.BloodFlow = 1f;
                            circulationBehaviour.IsPump = circulationBehaviour.WasInitiallyPumping;
                            limb.Person.Consciousness = 1f;
                            limb.PhysicalBehaviour.BurnProgress *= 0f;
                            
                            continue;
                        }

                        limb.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(rcpb.lifeS.GetLiquidID()), rcpb.limits.y);
                        Liquid.GetLiquid(rcpb.lifeS.GetLiquidID()).OnEnterLimb(limb);
                        limb.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(rcpb.lifeS.GetLiquidID()), rcpb.limits.y);
                    }                            
                })
            });
            revive.LabelWhenMultipleAreSelected = "Revive";
           
            var collapse = new ContextMenuButton(() => rcpb.collapsed == false, "collapse", "Collapse", "Collapse the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    rcpb.collapsed = true;
                    foreach (var limb in PB.Limbs)
                    {
                        limb.Health = 0f;
                        limb.CirculationBehaviour.IsPump = false;
                        limb.CirculationBehaviour.ForceSetAllLiquid(0f);
                        limb.BreakingThreshold = 0f;
                    }
                })
            });
            collapse.LabelWhenMultipleAreSelected = "Collapse";

            var zombify = new ContextMenuButton(() => rcpb.zombified == false, "zombify", "Zombify", "Zombify the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    rcpb.zombified = true;

                    foreach (var limb in PB.Limbs) {
                        limb.CirculationBehaviour.AddLiquid(Liquid.GetLiquid(rcpb.zombieS.GetLiquidID()), rcpb.limits.y);
                        Liquid.GetLiquid(rcpb.zombieS.GetLiquidID()).OnEnterLimb(limb);
                        limb.CirculationBehaviour.RemoveLiquid(Liquid.GetLiquid(rcpb.zombieS.GetLiquidID()), rcpb.limits.y);
                    }
                })
            });
            zombify.LabelWhenMultipleAreSelected = "Zombify";

            var disablePain = new ContextMenuButton(() => PB.IsAlive() && rcpb.NoPain == false, "disablePain", "Disable pain", "Disable the pain of the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    PB.PainLevel = 0.0f;
                    PB.ShockLevel = 0.0f;
                    rcpb.NoPain = true;
                })
            });
            disablePain.LabelWhenMultipleAreSelected = "Disable pain";

            var enablePain = new ContextMenuButton(() => PB.IsAlive() && rcpb.NoPain == true, "enablePain", "Enable pain", "Enable the pain of the selected human", new UnityAction[1]
            {
                (UnityAction) (() => rcpb.NoPain = false)
            });
            enablePain.LabelWhenMultipleAreSelected = "Enable pain";

            var disableDrowning = new ContextMenuButton(() => PB.IsAlive() && rcpb.NoDrown == false && rcpb.Suffocating == false, "disableDrowning", "Disable drowning", "Disable drowning of the selected human", new UnityAction[1]
            {
                (UnityAction) (() => rcpb.NoDrown = true)
            });
            disableDrowning.LabelWhenMultipleAreSelected = "Disable drowning";

            var enableDrowning = new ContextMenuButton(() => PB.IsAlive() && rcpb.NoDrown == true && rcpb.Suffocating == false, "enableDrowning", "Enable drowning", "Enable drowning of the selected human", new UnityAction[1]
            {
                (UnityAction) (() => rcpb.NoDrown = false)
            });
            enableDrowning.LabelWhenMultipleAreSelected = "Enable drowning";

            
            var suffocate = new ContextMenuButton(() => PB.IsAlive() && rcpb.NoDrown == false && rcpb.Suffocating == false, "suffocate", "Suffocate", "Suffocate the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    PB.OxygenLevel = 0f;
                    PB.PainLevel = 1f;
                    rcpb.Suffocating = true;
                })
            });
            suffocate.LabelWhenMultipleAreSelected = "Suffocate";

            var unsuffocate = new ContextMenuButton(() => PB.IsAlive() && rcpb.NoDrown == false && rcpb.Suffocating == true, "unSuffocate", "Unsuffocate", "Unsuffocate the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    PB.OxygenLevel = 1f;
                    rcpb.Suffocating = false;
                })
            });
            unsuffocate.LabelWhenMultipleAreSelected = "Unsuffocate";

            var invincible = new ContextMenuButton(() => rcpb.Invincibling == false, "invincible", "Make invincible", "Make the selected human invincible", new UnityAction[1]
            {
                (UnityAction) (() => rcpb.Invincibling = true)
            });
            invincible.LabelWhenMultipleAreSelected = "Make invincible";

            var uninvincible = new ContextMenuButton(() => rcpb.Invincibling == true, "unInvincible", "Remove invincibility", "Remove the invincibility of the selected human", new UnityAction[1]
            {
                (UnityAction) (() => {
                    rcpb.Invincibling = false;

                    foreach (var limb in PB.Limbs) {
                        limb.CirculationBehaviour.ClearLiquid();
                        limb.CirculationBehaviour.AddLiquid(limb.GetOriginalBloodType(), 1f);
                    }
                })
            });
            uninvincible.LabelWhenMultipleAreSelected = "Remove invincibility";

            List<ContextMenuButton> buttonsList = limb.PhysicalBehaviour.ContextMenuOptions.Buttons;

            buttonsList.Add(knockOut);
            buttonsList.Add(wakeUp);
            buttonsList.Add(collapse);
            buttonsList.Add(kill);
            buttonsList.Add(revive);
            buttonsList.Add(zombify);
            buttonsList.Add(disablePain);
            buttonsList.Add(enablePain);
            buttonsList.Add(disableDrowning);
            buttonsList.Add(enableDrowning);
            buttonsList.Add(suffocate);
            buttonsList.Add(unsuffocate);
            buttonsList.Add(invincible);
            buttonsList.Add(uninvincible);
        }
    }
}